@@ -9,7 +9,6 @@ import pathlib
 import pickle
 import random
 import re
-import shutil
 import subprocess
 import tempfile
 from abc import ABC, abstractmethod
@@ -67,7 +66,7 @@ VectorT = TypeVar("VectorT", bound="VectorBase")
 CompositeListElementT = TypeVar("CompositeListElementT")
 
 
-class HashableMixin:
+class Hashable:
     @property
     def hash_value(self) -> str:
         data_pickle = pickle.dumps(self)
@@ -75,7 +74,7 @@ class HashableMixin:
         return data_md5[:8]
 
 
-class MetaData(Dict[str, Union[str, int, float]], HashableMixin):
+class MetaData(Dict[str, Union[str, int, float]], Hashable):
     pass
 
 
@@ -560,7 +559,7 @@ def create_composite_image_sequence(
     return ElementSequence(composite_image_list)
 
 
-class TypeShapeTableMixin:
+class HasTypeShapeTable:
     def types(self) -> List[Type[ElementBase]]:
         return list(self.type_shape_table.keys())  # type: ignore
 
@@ -600,7 +599,7 @@ class TimeStampSequence(HasAList[float]):
 
 
 @dataclass
-class EpisodeData(TypeShapeTableMixin):
+class EpisodeData(HasTypeShapeTable, Hashable):
     sequence_dict: Dict[Type[ElementBase], ElementSequence[ElementBase]]
     metadata: MetaData
     time_stamp_seq: Optional[TimeStampSequence] = None
@@ -844,7 +843,7 @@ class EpisodeData(TypeShapeTableMixin):
 
 
 @dataclass(frozen=True)
-class BundleSpec(TypeShapeTableMixin):
+class BundleSpec(HasTypeShapeTable):
     n_episode: int
     n_untouch_episode: int
     n_average: int
@@ -887,7 +886,7 @@ _bundle_cache: Dict[Tuple[Path, Optional[str]], "EpisodeBundle"] = {}  # used Ep
 
 
 @dataclass
-class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
+class EpisodeBundle(HasAList[EpisodeData], HasTypeShapeTable):
     """Bundle of episode
     The collection of episodes.
     we call it 'bundle' because 'Dataset' is already used by pytorch
@@ -925,28 +924,42 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
     @classmethod
     def from_episodes(
         cls,
-        data_list: List[EpisodeData],
+        episode_list: List[EpisodeData],
         meta_data: Optional[MetaData] = None,
         shuffle: bool = True,
         n_untouch_episode: int = 5,
+        check_duplication: bool = True,
     ) -> "EpisodeBundle":
 
+        # check if episode data dupliation
+        if check_duplication:
+            hash_list = [e.hash_value for e in episode_list]
+            hash_set = set(hash_list)
+
+            n_list = len(hash_list)
+            n_set = len(hash_set)
+            assert (
+                n_list == n_set
+            ), "episode duplication found. list length {}, set length {}".format(n_list, n_set)
+
         if meta_data is None:
             meta_data = MetaData({})
 
-        set_types = set(functools.reduce(operator.add, [list(data.types()) for data in data_list]))
+        set_types = set(
+            functools.reduce(operator.add, [list(data.types()) for data in episode_list])
+        )
 
         n_type_appeared = len(set_types)
-        n_type_expected = len(data_list[0].types())
+        n_type_expected = len(episode_list[0].types())
         assert_equal_with_message(n_type_appeared, n_type_expected, "num of element type in bundle")
 
         untouch_episode_list = []
         if n_untouch_episode > 0:
-            interval = len(data_list) // n_untouch_episode
+            interval = len(episode_list) // n_untouch_episode
             indices_untouch = [interval * i for i in range(n_untouch_episode)]
             # sorted is necessary because pop changes index
             for idx in sorted(indices_untouch, reverse=True):
-                untouch_episode_list.append(data_list.pop(idx))
+                untouch_episode_list.append(episode_list.pop(idx))
 
         # use fixed random seed where it's scope is only in this function
         # The reason why I used fixed seed is to keep two different shuffled
@@ -956,9 +969,9 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         rn = random.Random()
         rn.seed(0)
         if shuffle:
-            rn.shuffle(data_list)
+            rn.shuffle(episode_list)
 
-        return cls(data_list, untouch_episode_list, meta_data)
+        return cls(episode_list, untouch_episode_list, meta_data)
 
     @classmethod
     def _load(cls, bundle_dir_path: pathlib.Path, postfix: Optional[str]) -> "EpisodeBundle":
@@ -982,15 +995,7 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         return bundle
 
     @classmethod
-    def load(
-        cls,
-        project_path: Path,
-        postfix: Optional[str] = None,
-        use_tar: bool = True,
-    ) -> "EpisodeBundle":
-        """load bundle
-        use_tar: default True. if True, load tar archive, otherwise load from a directory
-        """
+    def load(cls, project_path: Path, postfix: Optional[str] = None) -> "EpisodeBundle":
 
         if (project_path, postfix) not in _bundle_cache:
 
@@ -998,21 +1003,18 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
             if postfix is not None:
                 bundle_file_without_ext += "-{}".format(postfix)
 
-            if use_tar:
-                bundle_tar = bundle_file_without_ext + ".tar"
-                bundle_tar_path = project_path / bundle_tar
+            bundle_tar = bundle_file_without_ext + ".tar"
+            bundle_tar_path = project_path / bundle_tar
+            if not bundle_tar_path.exists():
+                raise FileNotFoundError("budle {} is not found".format(bundle_tar_path))
 
-                with tempfile.TemporaryDirectory() as tmp_dir:
-                    tmp_dir_path = pathlib.Path(tmp_dir)
-                    # TODO: use python's tarfile library
-                    subprocess.run(
-                        "cd {} && tar -xf {}".format(tmp_dir_path, bundle_tar_path), shell=True
-                    )
-                    bundle_dir_path = tmp_dir_path / bundle_file_without_ext
-                    bundle = cls._load(bundle_dir_path, postfix)
-
-            else:
-                bundle_dir_path = project_path / bundle_file_without_ext
+            with tempfile.TemporaryDirectory() as tmp_dir:
+                tmp_dir_path = pathlib.Path(tmp_dir)
+                # TODO: use python's tarfile library
+                subprocess.run(
+                    "cd {} && tar -xf {}".format(tmp_dir_path, bundle_tar_path), shell=True
+                )
+                bundle_dir_path = tmp_dir_path / bundle_file_without_ext
                 bundle = cls._load(bundle_dir_path, postfix)
 
             # because loading time of bundle is not negligible, we will cache the bundle with thep project_path and postfix key
@@ -1026,10 +1028,9 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
         self,
         project_path: Path,
         postfix: Optional[str] = None,
-        use_tar: bool = True,
+        exist_ok: bool = False,
     ) -> None:
         """dump the bundle
-        use_tar: defalut True. if True, save as tar archive, otherwise save as a directory
 
         NOTE: tar is great because it's immutable and no trouble when downloading from gdrive
         and it can be easily viewed on common file viewer.
@@ -1061,21 +1062,19 @@ class EpisodeBundle(HasAList[EpisodeData], TypeShapeTableMixin):
             with metadata_file_path.open(mode="w") as f:
                 json.dump(self.metadata, f)
 
-            if use_tar:
-                tarfile = bundle_file_without_ext + ".tar"
-                tarfile_full = project_path / tarfile
-                if tarfile_full.exists():
+            tarfile = bundle_file_without_ext + ".tar"
+            tarfile_full = project_path / tarfile
+            if tarfile_full.exists():
+                if exist_ok:
                     os.remove(tarfile_full)
+                else:
+                    raise FileExistsError(
+                        "Bundle file already exists. remove the file or use exist_ok=True."
+                    )
 
-                # TODO: using python tarfile is clean appoach. If get annoyed, please send a PR
-                cmd = "cd {} && tar cvf {} *".format(tmp_dir_path, tarfile_full)
-                subprocess.run(cmd, shell=True)
-            else:  # just move things to project directory
-                destination_path = project_path / bundle_file_without_ext
-                if destination_path.exists():
-                    shutil.rmtree(destination_path)
-                # https://bugs.python.org/issue34069
-                shutil.move(str(bundle_dir_path), str(destination_path))
+            # TODO: using python tarfile is clean appoach. If get annoyed, please send a PR
+            cmd = "cd {} && tar cvf {} *".format(tmp_dir_path, tarfile_full)
+            subprocess.run(cmd, shell=True)
 
         # extra dump just for debugging (the following info is not requried to load bundle)
         self.spec.dump(project_path, postfix)
